"""Provide the Aeripy class"""
import logging
from .rest_adapter import RestAdapter
from .models import *
from .endpoints import API_PATH
from .models import SystemInfo, School
from .util import snake_case_keys, camel_case_keys
from dotenv import load_dotenv


class Aeripy:
    """
    The Aeripy class provides access to the Aeries API.

    Instances of this class are the primary way to interact with the Aeries API.  To obtain and instance of this class:

    .. code-block:: python

        from aeripy import Aeripy

        aeries = Aeripy(
            hostname="demo.aeries.net/aeries/api",
            api_key="477abe9e7d27439681d62f4e0de1f5e1"
        )

    """
    def __init__(self, hostname: str,
                 api_key: str,
                 ver: str = 'v5',
                 ssl_verify: str = '',
                 logger: logging.Logger = None,
                 page_size: int = 5):
        """
        Initialize a :class:`Aeripy` instance.

        :param hostname: The base url of the Aeries SIS with the path /api on the end
        :param api_key: This api_key is found in Aeries->Security->API security. See Aeries documentation for details
        :param ver: default version is v5
        :param ssl_verify: The path to your SSL cert must be provided if your district uses SSL inspection
        :param logger: default logger
        :param page_size:

        """
        self._rest_adapter = RestAdapter(hostname, api_key, ver, ssl_verify, logger)
        self._page_size = page_size

    def get_system_info(self) -> SystemInfo:
        """
        Gets information about the target Aeries SIS.
        :return: SystemInfo
        """
        result = self._rest_adapter.get(endpoint=API_PATH["system_info"])
        sys_info = SystemInfo(**snake_case_keys(result.data))
        return sys_info

    def get_schools(self) -> List[School]:
        """
        Gets all the schools in the Aeries system.  To get info about one school, use get_school().
        :return: List[School], a list of all schools in Aeries
        """
        result = self._rest_adapter.get(endpoint=API_PATH['schools'])
        schools_list = [School(**snake_case_keys(datum)) for datum in result.data]
        return schools_list

    def get_school(self, school_code: int) -> School:
        """
        Gets info about a specific school.  If the schools does not exist an HTTP 404 error is returned

        :param school_code: Int, required
        :return: School
        """
        result = self._rest_adapter.get(endpoint=API_PATH['school'].format(school_code=school_code))
        school = School(**snake_case_keys(result.data))
        return school

    def get_terms(self, school_code: int) -> List[Term]:
        """
        Gets a list of terms for the school

        :param school_code: Int, required
        :return: List[Term]
        """
        result = self._rest_adapter.get(endpoint=API_PATH['terms'].format(school_code=school_code))
        terms_list = [Term(**snake_case_keys(datum)) for datum in result.data]
        return terms_list

    def get_bell_schedules(self, school_code: int, date: str = None) -> List[BellScheduleElement]:
        """
        Gets bell schedules for all schools. A date can be supplied to get the schedule for a specific date

        :param school_code: Int, required
        :param date: Str, optional, in the format mm-dd-yyyy
        :return: List[BellScheduleElement]
        """
        if date is not None:
            endpoint = API_PATH['bell_schedule_date'].format(school_code=school_code, date=date)
        else:
            endpoint = API_PATH['bell_schedule'].format(school_code=school_code)
        result = self._rest_adapter.get(endpoint=endpoint)
        bell_schedules = [BellScheduleElement(**snake_case_keys(datum)) for datum in result.data]
        return bell_schedules

    def get_bell_schedule(self, school_code: int, date: str) -> BellScheduleElement:
        """
        Gets bell schedule for individual school.  If the date is not a school day
        :param school_code: Int, required.
        :param date: Str, required, in the format "mm-dd-yyyy".
        :return: BellScheduleElement
        """
        result = self.get_bell_schedules(school_code, date)
        bell_schedule_list = BellScheduleElement(**snake_case_keys(result.data[0]))
        return bell_schedule_list

    def get_calendar(self, school_code: int) -> List[CalendarElement]:
        result = self._rest_adapter.get(endpoint=API_PATH["calendar"].format(school_code=school_code))
        calendar = [CalendarElement(**snake_case_keys(datum)) for datum in result.data]
        return calendar

    def get_absence_codes(self, school_code: int, absence_code: int = None) -> List[AbsenceCodeElement]:
        if absence_code is not None:
            endpoint = API_PATH['absence_code'].format(school_code=school_code, absence_code=absence_code)
        else:
            endpoint = API_PATH['absence_codes'].format(school_code=school_code)
        result = self._rest_adapter.get(endpoint=endpoint)
        absence_codes_list = [AbsenceCodeElement(**snake_case_keys(datum)) for datum in result.data]
        return absence_codes_list

    def get_absence_code(self, school_code: int, absence_code: int) -> AbsenceCodeElement:
        result = self._rest_adapter.get(endpoint=API_PATH['absence_code'].format(school_code=school_code,
                                                                                 absence_code=absence_code))
        absence_code = AbsenceCodeElement(*snake_case_keys(result.data))
        return absence_code

    def get_staff(self, staff_id: int = None) -> List[StaffElement]:
        """Return objects from a GET request to the ``staff`` endpoint.
        :param staff_id: Int, ID of staff to get. None to request all staff (default: ``None``).
        :return: All staff will be returned if no staff_id is supplied
        """
        if staff_id is not None:
            endpoint = API_PATH['staff_id'].format(staff_id=staff_id)
            result = self._rest_adapter.get(endpoint=endpoint)
            staff = StaffElement(**snake_case_keys(result.data))
        else:
            endpoint = API_PATH['staff']
            result = self._rest_adapter.get(endpoint=endpoint)
            staff = [StaffElement(**snake_case_keys(datum)) for datum in result.data]
        return staff

    def insert_staff(self, data: dict) -> StaffElement:
        """Inserts staff into the Aeries SIS using POST. After a successful request,
        this end point returns HTTP status code 201,
        and the response body contains the ``staff`` object that was just created.
        :param data: Dict, the data to create the staff with.
        :return:
        """
        result = self._rest_adapter.post(endpoint=API_PATH["staff"], data=camel_case_keys(data))
        staff = StaffElement(**snake_case_keys(result.data))
        return staff

    def update_staff(self, data: dict, staff_id: int = None) -> StaffElement:
        """
        Update staff using PUT.
        A 200 status will be returned if staff exists.
        A 201 status will be returned if staff was created.
        :param data: Dict, If property is omitted or null, it will be ignored.  An empty string is a valid value.
        Except in the case of LeaveDate, which will be nulled if it is omitted.
        :param staff_id: Int, If staff_id is supplied, but doesn't exist, and if auto-generate IDs IS NOT enabled,
        a new record will be created. If auto-generate IDs IS enabled, an error will be generated
        :return:
        """
        if staff_id is None:
            staff_id = data.get("staff_id")
        result = self._rest_adapter.put(endpoint=API_PATH["staff_id"].format(staff_id=staff_id), data=camel_case_keys(data))
        staff = StaffElement(**snake_case_keys(result.data))
        return staff



import re
from typing import Any, Dict

_re_camel_to_snake = re.compile(r"([a-z0-9](?=[A-Z])|[A-Z](?=[A-Z][a-z]))")


def camel_to_snake(name: str) -> str:
    return _re_camel_to_snake.sub(r"\1_", name).lower()


def snake_to_camel(name: str) -> str:
    return ''.join([word.replace(word[0], word[0].upper()) for word in name.split('_')])


def snake_case_keys(dictionary: Dict[str, Any]) -> Dict[str, Any]:
    return {camel_to_snake(k): v for k, v in dictionary.items()}


def camel_case_keys(dictionary: Dict[str, Any]) -> Dict[str, Any]:
    return {snake_to_camel(k): v for k, v in dictionary.items()}



import requests
import logging
from typing import List, Dict, Union
from json import JSONDecodeError
from aeripy.exceptions import AeripyException
from aeripy.models import Result


class RestAdapter:
    def __init__(self, hostname: str,
                 api_key: str, ver: str = 'v5',
                 ssl_verify: str = '',
                 logger: logging.Logger = None):
        """
        Constructor for RestAdapter
        :param hostname: URL of the Aeries instance. Required
        :param api_key: API key of the Aeries instance. Required
        :param ver: v5 is the default
        :param ssl_verify: True by default, but can also take a PEM path for testing
        :param logger: (optional) If your app has a logger, pass it here
        """
        self._logger = logger or logging.getLogger(__name__)
        self.url = f"https://{hostname}/{ver}"
        self._api_key = api_key
        self._ssl_verify = ssl_verify
        if not ssl_verify:
            # noinspection PyUnresolvedReferences
            requests.packages.urllib3.disable_warnings()

    def _do(self, http_method: str, endpoint: str, ep_params: Dict = None, data: Dict = None) -> Result:
        """
        Private method for get(), post, delete() methods
        :param http_method: GET, POST, DELETE
        :param endpoint: URL endpoint as a string
        :param ep_params: Dictionary of endpoint parameters (optional)
        :param data: Dictionary of data to pass to Aeripy (optional)
        :return: a Result object
        """
        full_url = self.url + endpoint
        headers = {"AERIES-CERT": self._api_key}
        log_line_pre = f"method{http_method}, url={full_url}, params={ep_params}"
        log_line_post = ', '.join((log_line_pre, "success={}, status_code={}, message={}"))

        # Log HTTP params and perform an HTTP request, catching and re-raising any exceptions
        try:
            self._logger.debug(msg=log_line_pre)
            response = requests.request(method=http_method, url=full_url, verify=self._ssl_verify,
                                        headers=headers, params=ep_params, json=data)
        except requests.exceptions.RequestException as e:
            self._logger.error(msg=(str(e)))
            raise AeripyException("Request failed") from e

        # Deserialize JSON output to Python object, or return failed Result on exception
        try:
            data_out = response.json()
        except (ValueError, TypeError, JSONDecodeError) as e:
            self._logger.error(msg=log_line_post.format(False, None, e))
            raise AeripyException("Bad JSON in response") from e

        # If status_code in 200-299 range, return success Result with data, otherwise raise exception
        is_success = 299 >= response.status_code >= 200     # 200 to 299 is OK
        log_line = log_line_post.format(is_success, response.status_code, response.reason)
        if is_success:
            self._logger.debug(msg=log_line)
            return Result(response.status_code, headers=response.headers, message=response.reason, data=data_out)
        self._logger.error(msg=log_line)
        raise AeripyException(f"{response.status_code}: {response.reason}")

    def get(self, endpoint: str, ep_params: Dict = None) -> Result:
        return self._do(http_method='GET', endpoint=endpoint, ep_params=ep_params)

    def post(self, endpoint: str, ep_params: Dict = None, data: Dict = None) -> Result:
        return self._do(http_method='POST', endpoint=endpoint, ep_params=ep_params, data=data)

    def put(self, endpoint: str, ep_params: Dict = None, data: Dict = None) -> Result:
        return self._do(http_method='PUT', endpoint=endpoint, ep_params=ep_params, data=data)

    def delete(self, endpoint: str, ep_params: Dict = None, data: Dict = None) -> Result:
        return self._do(http_method='DELETE', endpoint=endpoint, ep_params=ep_params, data=data)



"""List of Aeries API endpoints
https://support.aeries.com/support/solutions/articles/14000070569-api-quick-reference-guide"""

API_PATH = {
    "system_info":          "/systeminfo",
    "schools":              "/schools",
    "school":               "/schools/{school_code}",
    "terms":                "/schools/{school_code}/terms",
    "bell_schedule":        "/schools/{school_code}/BellSchedule",
    "bell_schedule_date":   "/schools/{school_code}/BellSchedule/date/{date}",
    "calendar":             "/schools/{school_code}/calendar",
    "absence_codes":        "/schools/{school_code}/AbsenceCodes",
    "absence_code":         "/schools/{school_code}/AbsenceCodes/{AbsenceCode}",
    "staff":                "/staff/",
    "staff_id":             "/staff/{staff_id}",
    "staff_hrid":           "/staff/hrid/{hr_id}",
    "assignments":          "/staff/{staff_id}/assignments",
    "assignments_type":     "/staff/{staff_id}/assignments/{assignment_type}",
    "assignments_sequence": "/staff/{staff_id}/assignments/{assignment_type}/{sequence_number}"
}




# Always prefer setuptools over distutils
from setuptools import setup

# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# This call to setup() does all the work
setup(
    name="aeripy",
    version="0.2.13",
    description="Aeries API Python Adapter",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://aeripy.readthedocs.io/",
    author="Ryan Aguilar",
    author_email="ryanaguilar323@gmail.com",
    license="Simplified BSD License",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent"
    ],
    packages=["aeripy"],
    include_package_data=True,
    install_requires=["requests"]
)
